#include <stdio.h>
#include <math.h>
/* eq_grado_2.c : file sorgente di un programma che calcola le soluzioni
                  di un'equazione di secondo grado in una sola incognita,
                  distinguendo i casi in cui le soluzioni sono:
                    1 - reali e distinte 
                    2 - reali e coincidenti
                    3 - complesse con parte immaginaria non nulla */
/* Definizione dei prototipi delle funzioni */
int risolvi_eq(double a, double b, double c, double *x1, double *x2);
/* ### */
int main()
{
  /* Definizione delle variabili:
     a,b,c = coefficienti dell'equazione di secondo grado
                a*x^2 + b*x + c = 0
     x1,x2 = se b^2 - 4*a*c >= 0 allora x1 e x2 sono poste uguali alle
             soluzioni dell'equazione di secondo grado precedente;
	     se b^2 - 4*a*c < 0 allora x1 e x2 sono poste rispettivamente
	     uguali alla parte reale e alla parte immaginaria delle
             soluzioni dell'equazione di secondo grado precedente;
     flag = variabile cosiddetta di "flag" (=bandiera) che a seconda del
            suo valore distingue tra diverse situazioni:
	            1 - il segno di b^2 - 4*a*c e' positivo
                    0 - il valore di b^2 - 4*a*c e' nullo
                   -1 - il segno di b^2 - 4*a*c e' negativo */
  int flag;
  double a, b, c, x1, x2;
  /* Fase di input dei coefficienti a, b, c */
  printf("Dammi tre coefficienti a, b, c:\n");
  do {
    printf("  (ricordati che a deve essere diverso da zero)\n");
    scanf("%lf %lf %lf", &a, &b, &c);
  } while (a == 0.);
  /* Risolviamo l'equazione di secondo grado, assegnando alle
     variabili x1 e x2 le soluzioni e a flag il valore 1, 0, -1
     a seconda del segno di b^2 - 4*a*c */
  flag = risolvi_eq(a, b, c, &x1, &x2);
  /* Distinguiamo i diversi casi con le istruzioni
     switch ... case ... case ... default ... 
     (potremmo farlo anche con
      if ... else if ... else if ... else ... */
  switch(flag) {
  case 1:
    /* Nel caso flag=1 scriviamo un messaggio che dica che ci sono due
       soluzioni reali e distinte */
    printf("Esistono due soluzioni reali e distinte:\n");
    /* Stampiamo i valori di x1 e x2 */
    printf("  x1 = %lf ; x2 = %lf\n", x1, x2);
    /* Attenzione a non dimenticare l'istruzione break, altrimenti
       vengono eseguite anche le istruzioni seguenti fino al carattere } */
    break;
  case 0:
    /* Nel caso flag=0 scriviamo un messaggio che dica che ci sono due
       soluzioni reali coincidenti */
    printf("Esistono due soluzioni reali coincidenti:\n");
    /* Stampiamo il valore di x1 */
    printf("  x1 = x2 = %lf\n", x1);
    /* Attenzione a non dimenticare l'istruzione break, altrimenti
       vengono eseguite anche le istruzioni seguenti fino al carattere } */
    break;
  default:
    /* Nel caso flag=-1 scriviamo un messaggio che dica che ci sono due
       soluzioni complesse */
    printf("Esistono due soluzioni complesse:\n");
    /* Stampiamo le soluzioni nella forma x1 +- i*x2 */
    printf("  %lf +- i*%lf\n", x1, x2);
    /* Questo e' l'ultimo caso, quindi non e' necessaria l'istruzione break */
  }
  exit(0);
}
/* ### */
int risolvi_eq(a, b, c, x1, x2)
     double a, b, c, *x1, *x2;
{
  int flag;
  double Delta;
  /* Poniamo Delta = b^2 - 4*a*c */
  Delta = b*b - 4.*a*c;
  /* Distinguiamo i diversi casi a seconda del segno di Delta */
  if(Delta > 0.) {
    /* Siamo nel caso delle due radici reali e distinte:
       calcoliamo x1 ... */
    *x1 = -0.5 * ( b + sqrt(Delta) ) / a;
    /* ... e x2 */
    *x2 = -0.5 * ( b - sqrt(Delta) ) / a;
    /* Ricordiamoci di inizializzare flag */
    flag = 1;
  }
  else if(Delta == 0.) {
    /* Siamo nel caso delle due radici reali coincidenti:
       calcoliamo x1 e x2 */
    *x1 = *x2 = -0.5 * b / a;
    /* Ricordiamoci di inizializzare flag */
    flag = 0;
  }
  else {
    /* Siamo nel caso delle due radici complesse:
       poniamo x1 uguale alla parte reale delle soluzioni ... */
    *x1 = -0.5 * b / a;
    /* ... e poniamo x2 uguale alla parte immaginaria delle soluzioni */
    *x2 = 0.5 * sqrt(-Delta) / a;
    /* Ricordiamoci di inizializzare flag */
    flag = -1;
  }
  /* Usciamo dalla funzione restituendo il valore di flag */
  return(flag);
}
