#include <stdio.h>
#include <math.h>
/* logaritmo.c : file sorgente di un programma che effettua il calcolo
                 del logaritmo di un numero compreso tra 0 e 2 con un
                 procedimento iterativo che migliora via via
                 l'approssimazione del risultato.  L'algoritmo di
                 calcolo (ovviamente) fa uso dell'espansione in serie
                 di potenze della funzione log(1+t) con t prossimo a
                 zero. */
/* ### */
/* La prossima definizione consente di cambiare facilmente il tipo di
   variabili con cui si effettua il calcolo (da double a float o il
   viceversa oppure da double ad altri tipi) */
#define DOUBLE double
/* Attenzione!
   Se si vuole utilizzare il tipo float anziche' il tipo double bisogna
   togliere le l dai formati di conversione (cioe' occorre utilizzare
   f o e anziche' lf o le, rispettivamente).
   Se si vuole utilizzare il tipo long double anziche' il tipo double
   bisogna sostituire le l con delle L nei formati di conversione (cioe'
   occorre utilizzare Lf o Le anziche' lf o le, rispettivamente). */
/* Definizione dei prototipi delle funzioni */
DOUBLE logmio(DOUBLE x);
/* ### */
int main()
{
  /* Definizione delle variabili:
     x = numero del quale vogliamo calcolare il logaritmo
     y = numero che rappresenta l'approssimazione calcolata del
         logaritmo di x */
  DOUBLE x, y;
  /* Inizio delle istruzioni eseguibili */
  /* Input di x */
  do {
    printf("Numero x (0 < x <= 2) di cui vuoi ");
    printf("calcolare il logaritmo?\n");
    scanf("%lf", &x);
  } while (x <= 0. || x > 2.);
  /* Calcolo del logaritmo utilizzando l'espansione in serie di
     potenze scritta nella function logmio */
  y = logmio(x);
  /* Stampa del risultato */
  printf("Il valore y da me calcolato del logaritmo di x e' : %20.16lf\n", y);
  /* Scrivo l'errore assoluto finale rispetto al risultato dato dalla 
     funzione di libreria log */
  printf("Il valore assoluto di y - log(x) e' : ");
  printf("%10.3le\n", fabs( y - log(x) ) );
  /* Conclusione delle istruzioni eseguibili */
  exit(0);
}
/* ### */
DOUBLE logmio(x)
    DOUBLE x;
    /* Nota: questa function e' stata scritta in modo tale che
             i termini dell'espansione in serie di potenze vengano
	     sommati a due a due. Questa scelta implica che la
	     scrittura della funzione sia piu' lunga, ma la condizione
	     di arresto del calcolo numerico dell'espansione in serie
	     puo' essere raggiunto prima per valori di x maggiori di 1,
	     poiche' si sfrutta il fatto che quando x>1 i termini di
	     indice dispari e quelli di indice pari si compensano
	     parzialmente */
{
  /* Definizione della variabile contenente il risultato */
  DOUBLE ris;
  /* Definizione della variabile contenente il risultato come
     calcolato all'approssimazione precedente */
  DOUBLE risprec;
  /* Definizione di un contatore i */
  int i;
  /* Definizione di una variabile che contiene le potenze di (x - 1) */
  DOUBLE potx;
  /* Definizione di una variabile che contiene, ad ogni iterazione,
     l'addendo che viene aggiunto alle somme parziali della serie */
  DOUBLE addcoef;
  /* Inizio delle istruzioni eseguibili */
  /* Azzeramento del risultato */
  ris = 0.;
  /* Azzeramento del contatore i */  
  i = 0;
  /* Decrementiamo di 1 il valore di x (nello sviluppo in serie compaiono
     solo potenze di x - 1 e non di x) */
  x -= 1.;
  /* Poniamo potx uguale a 1 (cioe' x elevato alla 0) */
  potx = 1.;
  /* Inizio di un ciclo da eseguire finche' il risultato numerico non
     cambia */
  do {
    /* Aggiorniamo il risultato precedente ponendolo uguale a ris (cioe'
       il risultato calcolato all'iterazione precedente del ciclo) */
    risprec = ris;
    /* Incrementiamo di 1 il contatore i */
    i++;
    /* Aggiorniamo il valore di potx ponendolo uguale a x elevato alla i
       (si ricordi che prima dell'istruzione seguente potx = x^(i-1) */
    potx *= x;
    /* Poniamo addcoef uguale al termine generale i-esimo (con segno
       positivo! Si noti che adesso i e' dispari) della serie di
       potenze che definisce il logaritmo */
    addcoef =  1. / i * potx;
    /* Incrementiamo di 1 il contatore i */
    i++;
    /* Aggiorniamo il valore di potx ponendolo uguale a x elevato alla i
       (si ricordi che prima dell'istruzione seguente potx = x^(i-1) */
    potx *= x;
    /* Sottraiamo ad addcoef il nuovo termine generale i-esimo (che ha
       segno negativo! Si noti che adesso i e' pari) della serie di
       potenze che definisce il logaritmo */
    addcoef -=  1. / i * potx;
    /* Aggiorniamo il risultato aggiungendo la correzione dovuta
       ad addcoef */
    ris += addcoef;
    /* Eseguiamo il test di uscita dal ciclo: se il risultato e'
       diverso da quello all'iterazione precedente continuiamo ad
       iterare il ciclo */
  } while(ris != risprec);
  /* restituiamo il valore del risultato alla funzione chiamante */
  return(ris);
}
