#include <stdio.h>
/* trig_iter.c : file sorgente di un programma che calcola in modo
                 iterativo l'espansione di cos(nx) e sen(nx) in
                 funzione delle potenze di cos x e sen x */
/* ### */
/* La prossima definizione consente di cambiare facilmente il
   coefficiente intero n massimo delle funzioni cos(nx) e sen(nx) di
   cui vogliamo calcolare gli sviluppi */
#define NMAX 25
/* Definizione dei prototipi delle funzioni */
void cossenn(int cosi[], int seni[], int n);
void stampasvil(int svil[], int n);
/* ### */
int main()
{
  /* Definizione delle variabili:
     n = coefficiente intero di cos(nx) e sen(nx)
     cosn = vettore di coefficienti interi che rappresenta
            lo sviluppo delle funzioni cos(nx)
     senn = vettore di coefficienti interi che rappresenta
            lo sviluppo delle funzioni sen(nx) */
  int n, cosn[NMAX+1], senn[NMAX+1];
  /* Fase di input del coefficiente intero n */
  do {
    printf("Dammi un numero positivo (<=%d): ", NMAX);
    scanf("%d", &n);
  } while (n <= 0  ||  n > NMAX);
  /* Calcoliamo simultaneamente gli sviluppi di cos(nx) e sen(nx) */
  cossenn(cosn, senn, n);
  /* Stampiamo "cos nx =  " */
  printf("cos %dx =", n);
  /* Stampiamo lo sviluppo descritto dal vettore cosn */
  stampasvil(cosn, n);
  /* Stampiamo "sen nx =  " */
  printf("sen %dx =", n);
  /* Stampiamo lo sviluppo descritto dal vettore senn */
  stampasvil(senn, n);
  exit(0);
}
/* ### */
void cossenn(cosi, seni, n)
     int cosi[];
     int seni[];
     int n;
    /* Nota: questa function e' stata scritta in modo tale che il
             calcolo dello sviluppo di cos(nx) e sen(nx) sia
             iterativo */
{
  /* Definizione di due contatori i e j */
  int i, j;
  /* Definizione di cosimen1 e senimen1 che sono dei vettori di al
     piu' n coefficienti interi che rappresentano lo sviluppo delle
     funzioni cos((i-1)x) o sen((i-1)x) (con i che varia da 2 a n) */
  int cosimen1[NMAX], senimen1[NMAX];
  /* Definisco i valori iniziali dell'iterazione, cioe' gli sviluppi di
     cos(ix) e sen(ix) quando i=1:
     scriviamo nel vettore cosi lo sviluppo di cos(x) ... */
  cosi[0] = 1;
  cosi[1] = 0;
  /* ... E scriviamo nel vettore seni lo sviluppo di sen(x) */
  seni[0] = 0;
  seni[1] = 1;
  /* Eseguiamo un ciclo sul contatore i che va da 2 a n in modo da
     calcolare gli sviluppi di cos(ix) e sen(ix) una volta noti
     quelli di cos((i-1)x) e sen((i-1)x) */
  for(i=2; i<=n; i++) {
    /* Copio in cosimen1 lo sviluppo rappresentato nel vettore cosi */
    for(j=0; j<i; j++) {
      cosimen1[j] = cosi[j];
    }
    /* Copio in senimen1 lo sviluppo rappresentato nel vettore seni */
    for(j=0; j<i; j++) {
      senimen1[j] = seni[j];
    }
    /* Azzero lo sviluppo rappresentato nel vettore cosi */
    for(j=0; j<=i; j++) {
      cosi[j] = 0;
    }
    /* Eseguiamo un ciclo sul contatore j che va da 0 a i-1 in modo da
       riportare nel vettore cosi il prodotto di cos(x)*cos((i-1)x) */
    for(j=0; j<i; j++) {
      cosi[j] = cosimen1[j];
    }
    /* Eseguiamo un ciclo sul contatore j che va da 1 a i in modo da
       aggiungere allo sviluppo precedente nel vettore cosi il
       prodotto di -sen(x)*sen((i-1)x) */
    for(j=1; j<=i; j++) {
      cosi[j] -= senimen1[j-1];
    }
    /* Azzero lo sviluppo rappresentato nel vettore seni */
    for(j=0; j<=i; j++) {
      seni[j] = 0;
    }
    /* Eseguiamo un ciclo sul contatore j che va da 1 a i in modo da
       riportare nel vettore seni il prodotto di sen(x)*cos((i-1)x) */
    for(j=1; j<=i; j++) {
      seni[j] = cosimen1[j-1];
    }
    /* Eseguiamo un ciclo sul contatore j che va da 0 a i-1 in modo da
       aggiungere allo sviluppo precedente nel vettore seni il
       prodotto di cos(x)*sen((i-1)x) */
    for(j=0; j<i; j++) {
      seni[j] += senimen1[j];
    }
  }
  /* Uscita dalla funzione */
  return;
}
/* ### */
void stampasvil(svil, n)
     int svil[];
     int n;
{
  /* Definizione di un contatore i */
  int i;
  /* Definizione di una variabile intera nscritti che conta il numero
     di termini scritti dell'espansione */
  int nscritti;
  /* Azzero nscritti */
  nscritti = 0;
  /* Ciclo su i che va da 0 a n per scrivere tutti i termini dello
     sviluppo */
  for(i=0;i<=n;i++) {
    /* Stampiamo un termine solo se svil[i] e' diverso da zero */
    if(svil[i] != 0) {
      /* Ogni 2 termini scritti dell'espansione stampiamo un ritorno a capo
	 e qualche spazio */
      if(nscritti%2 == 0)  printf("\n   ");
      /* Se nscritti>0 e svil[i]>0 stampiamo il segno + */
      if(nscritti > 0  &&  svil[i] > 0)  printf("+");
      /* Stampiamo l'i-esimo coefficiente dello sviluppo */
      printf("%d ", svil[i]);
      /* Se i<n stampiamo cos^n-i x */
      if(i<n)  printf("cos^%d x ", n-i);
      /* Se i>0 stampiamo sen^i x */
      if(i>0)  printf("sen^%d x ", i);
      /* Se il termine che abbiamo stampato non e' l'ultimo della riga
	 stampiamo uno spazio per separare un termine rispetto
	 all'altro */
      if(nscritti%2 != 1)  printf(" ");
      /* Incrementiamo il numero di termini scritti */
      nscritti++;
    }
  }
  /* Stampa il ritorno a capo */
  printf("\n");
  return;
}
